@@ -101,6 +101,7 @@ int run_check(char *processed_command, char **ret, char **err) {
     pid_t pid;
     int pipe_stdout[2], pipe_stderr[2], pipe_rwe[3];
     int retval;
+    sigset_t mask;
 
 #ifdef EMBEDDEDPERL
     retval = run_epn_check(processed_command, ret, err);
@@ -109,8 +110,12 @@ int run_check(char *processed_command, char **ret, char **err) {
     }
 #endif
 
-    /* check for check execution method (shell or execvp) */
-    if(!strpbrk(processed_command,"!$^&*()~[]|{};<>?`\"'")) {
+    /* check for check execution method (shell or execvp)
+     * command line does not have to contain shell meta characters
+     * and cmd must begin with a /. Otherwise "BLAH=BLUB cmd" would lead
+     * to file not found errors
+     */
+    if((*processed_command == '/' || *processed_command == '.') && !strpbrk(processed_command,"!$^&*()~[]\\|{};<>?`\"'")) {
         /* use the fast execvp when there are no shell characters */
         gm_log( GM_LOG_TRACE, "using execvp, no shell characters found\n" );
 
@@ -131,6 +136,10 @@ int run_check(char *processed_command, char **ret, char **err) {
             _exit(STATE_UNKNOWN);
         }
         else if(!pid){
+            /* remove all customn signal handler */
+            sigfillset(&mask);
+            sigprocmask(SIG_UNBLOCK, &mask, NULL);
+
             /* child process */
             if((dup2(pipe_stdout[1],STDOUT_FILENO)<0)){
                 gm_log( GM_LOG_ERROR, "dup2 error\n");
@@ -217,7 +226,6 @@ int execute_safe_command(gm_job_t * exec_job, int fork_exec, char * identifier)
     char *plugin_output, *plugin_error;
     char *bufdup;
     char buffer[GM_BUFFERSIZE], buf_error[GM_BUFFERSIZE];
-    sigset_t mask;
     struct timeval start_time,end_time;
     pid_t pid    = 0;
     buffer[0]    = '\x0';
@@ -225,6 +233,14 @@ int execute_safe_command(gm_job_t * exec_job, int fork_exec, char * identifier)
 
     gm_log( GM_LOG_TRACE, "execute_safe_command()\n" );
 
+    // mark all filehandles to close on exec
+    int x;
+    for(x = 0; x<=64; x++)
+        fcntl(x, F_SETFD, FD_CLOEXEC);
+
+    signal(SIGTERM, SIG_DFL);
+    signal(SIGINT, SIG_DFL);
+
     if(exec_job->start_time.tv_sec == 0) {
         gettimeofday(&start_time,NULL);
         exec_job->start_time = start_time;
@@ -254,10 +270,6 @@ int execute_safe_command(gm_job_t * exec_job, int fork_exec, char * identifier)
         setpgid(0,0);
         pid = getpid();
 
-        /* remove all customn signal handler */
-        sigfillset(&mask);
-        sigprocmask(SIG_UNBLOCK, &mask, NULL);
-
         if( fork_exec == GM_ENABLED ) {
             close(pipe_stdout[0]);
             close(pipe_stderr[0]);
@@ -328,11 +340,7 @@ int execute_safe_command(gm_job_t * exec_job, int fork_exec, char * identifier)
         else if(return_code >= 128 && return_code < 144) {
             char * signame = nr2signal((int)(return_code-128));
             bufdup = strdup(buffer);
-            snprintf( buffer, sizeof( buffer )-1, "CRITICAL: Return code of %d is out of bounds. Plugin exited by signal %s. (worker: %s)", (int)(return_code), signame, identifier);
-            if(strlen(bufdup) > 0) {
-                strncat(buffer, "\\n", (sizeof(buffer)-1));
-                strncat(buffer, bufdup, (sizeof(buffer)-1));
-            }
+            snprintf( buffer, sizeof( buffer )-1, "CRITICAL: Return code of %d is out of bounds. Plugin exited by signal %s. (worker: %s)\\n%s", (int)(return_code), signame, identifier, bufdup);
             return_code = STATE_CRITICAL;
             free(bufdup);
             free(signame);
@@ -383,10 +391,11 @@ int execute_safe_command(gm_job_t * exec_job, int fork_exec, char * identifier)
 
 /* called when check runs into timeout */
 void check_alarm_handler(int sig) {
+    pid_t pid;
     int retval;
-    pid_t pid = getpid();
 
     gm_log( GM_LOG_TRACE, "check_alarm_handler(%i)\n", sig );
+    pid = getpid();
     if(current_job != NULL && mod_gm_opt->fork_on_exec == GM_DISABLED) {
         /* create a useful log message*/
         if ( !strcmp( current_job->type, "service" ) ) {
@@ -402,30 +411,57 @@ void check_alarm_handler(int sig) {
         gearman_job_send_complete(current_gearman_job, NULL, 0);
     }
 
-    if(current_child_pid > 0) {
-        gm_log( GM_LOG_TRACE, "send SIGINT to %d\n", current_child_pid);
-        signal(SIGINT, SIG_IGN);
-        kill(-current_child_pid, SIGINT);
-        sleep(1);
-        if(waitpid(pid,&retval,0)!=pid)
-            return;
-        if(pid_alive(current_child_pid)) {
-            gm_log( GM_LOG_TRACE, "send SIGKILL to %d\n", current_child_pid);
-            kill(-current_child_pid, SIGKILL);
-        }
-    } else {
-        signal(SIGINT, SIG_IGN);
-        gm_log( GM_LOG_TRACE, "send SIGINT to %d\n", pid);
-        kill(-pid, SIGINT);
-        signal(SIGINT, SIG_DFL);
-        sleep(1);
+    signal(SIGTERM, SIG_IGN);
+    gm_log( GM_LOG_TRACE, "send SIGTERM to %d\n", pid);
+    kill(-pid, SIGTERM);
+    kill(pid, SIGTERM);
+    signal(SIGTERM, SIG_DFL);
+    sleep(1);
+
+    signal(SIGINT, SIG_IGN);
+    gm_log( GM_LOG_TRACE, "send SIGINT to %d\n", pid);
+    kill(-pid, SIGINT);
+    kill(pid, SIGINT);
+    signal(SIGINT, SIG_DFL);
+    sleep(1);
+
+    // skip sigkill in test mode
+    if(getenv("MODGEARMANTEST") == NULL) {
         gm_log( GM_LOG_TRACE, "send SIGKILL to %d\n", pid);
         kill(-pid, SIGKILL);
+        kill(pid, SIGKILL);
     }
 
     return;
 }
 
+/* send kill to all forked processes */
+void kill_child_checks(void) {
+    int retval;
+    pid_t pid;
+
+    signal(SIGINT, SIG_IGN);
+    pid = getpid();
+    if(current_child_pid > 0 && current_child_pid != pid) {
+        gm_log( GM_LOG_TRACE, "kill_child_checks(): send SIGINT to %d\n", current_child_pid);
+        kill(-current_child_pid, SIGINT);
+        kill(current_child_pid, SIGINT);
+        sleep(1);
+        if(waitpid(current_child_pid,&retval,WNOHANG)!=0) {
+            signal(SIGINT, SIG_DFL);
+            return;
+        }
+        if(pid_alive(current_child_pid)) {
+            gm_log( GM_LOG_TRACE, "kill_child_checks(): send SIGKILL to %d\n", current_child_pid);
+            kill(current_child_pid, SIGKILL);
+        }
+    }
+    gm_log( GM_LOG_TRACE, "send SIGINT to %d\n", pid);
+    kill(0, SIGINT);
+    signal(SIGINT, SIG_DFL);
+    return;
+}
+
 
 void send_timeout_result(gm_job_t * exec_job) {
     struct timeval end_time;
@@ -437,7 +473,7 @@ void send_timeout_result(gm_job_t * exec_job) {
     gettimeofday(&end_time, NULL);
     exec_job->finish_time = end_time;
 
-    exec_job->return_code   = 2;
+    exec_job->return_code   = mod_gm_opt->timeout_return;
     exec_job->early_timeout = 1;
     if ( !strcmp( exec_job->type, "service" ) )
         snprintf( buffer, sizeof( buffer ) -1, "(Service Check Timed Out On Worker: %s)\n", mod_gm_opt->identifier);
