@@ -237,6 +237,7 @@ int set_default_options(mod_gm_opt_t *opt) {
     opt->perfdata_mode      = GM_PERFDATA_OVERWRITE;
     opt->use_uniq_jobs      = GM_ENABLED;
     opt->do_hostchecks      = GM_ENABLED;
+    opt->route_eventhandler_like_checks = GM_DISABLED;
     opt->hosts              = GM_DISABLED;
     opt->services           = GM_DISABLED;
     opt->events             = GM_DISABLED;
@@ -260,6 +261,7 @@ int set_default_options(mod_gm_opt_t *opt) {
     opt->dup_results_are_passive = GM_ENABLED;
     opt->orphan_host_checks      = GM_ENABLED;
     opt->orphan_service_checks   = GM_ENABLED;
+    opt->orphan_return           = 2;
     opt->accept_clear_results    = GM_DISABLED;
     opt->has_starttime      = FALSE;
     opt->has_finishtime     = FALSE;
@@ -404,7 +406,9 @@ int parse_args_line(mod_gm_opt_t *opt, char * arg, int recursion_level) {
     }
 
     /* debug-result */
-    else if ( !strcmp( key, "debug-result" ) ) {
+    else if (   !strcmp( key, "debug-result" )
+             || !strcmp( key, "debug_result" )
+            ) {
         opt->debug_result = parse_yes_or_no(value, GM_ENABLED);
         return(GM_OK);
     }
@@ -427,6 +431,12 @@ int parse_args_line(mod_gm_opt_t *opt, char * arg, int recursion_level) {
         return(GM_OK);
     }
 
+    /* route_eventhandler_like_checks */
+    else if ( !strcmp( key, "route_eventhandler_like_checks" ) ) {
+        opt->route_eventhandler_like_checks = parse_yes_or_no(value, GM_ENABLED);
+        return(GM_OK);
+    }
+
     /* active */
     else if ( !strcmp( key, "active" ) ) {
         opt->active = parse_yes_or_no(value, GM_ENABLED);
@@ -688,11 +698,38 @@ int parse_args_line(mod_gm_opt_t *opt, char * arg, int recursion_level) {
         if(opt->spawn_rate < 0) { opt->spawn_rate = GM_DEFAULT_SPAWN_RATE; }
     }
 
+    /* load limit 1min */
+    else if ( !strcmp( key, "load_limit1" ) ) {
+        opt->load_limit1 = atof( value );
+        if(opt->load_limit1 < 0) { opt->load_limit1 = 0; }
+    }
+    /* load limit 5min */
+    else if ( !strcmp( key, "load_limit5" ) ) {
+        opt->load_limit5 = atof( value );
+        if(opt->load_limit5 < 0) { opt->load_limit5 = 0; }
+    }
+    /* load limit 15min */
+    else if ( !strcmp( key, "load_limit15" ) ) {
+        opt->load_limit15 = atof( value );
+        if(opt->load_limit15 < 0) { opt->load_limit15 = 0; }
+    }
+
     /* timeout_return */
     else if ( !strcmp( key, "timeout_return" ) ) {
         opt->timeout_return = atoi( value );
-        if(opt->timeout_return < 0) { opt->timeout_return = 2; }
-        if(opt->timeout_return > 3) { opt->timeout_return = 2; }
+        if(opt->timeout_return < 0 || opt->timeout_return > 3) {
+            gm_log( GM_LOG_INFO, "Warning: unknown timeout_return: %d\n", opt->timeout_return );
+            opt->timeout_return = 2;
+        }
+    }
+
+    /* orphan_return */
+    else if ( !strcmp( key, "orphan_return" ) ) {
+        opt->orphan_return = atoi( value );
+        if(opt->orphan_return < 0 || opt->orphan_return > 3) {
+            gm_log( GM_LOG_INFO, "Warning: unknown orphan_return: %d\n", opt->orphan_return );
+            opt->orphan_return = 2;
+        }
     }
 
     /* perfdata_mode */
@@ -899,105 +936,106 @@ void dumpconfig(mod_gm_opt_t *opt, int mode) {
     int j=0;
     gm_log( GM_LOG_DEBUG, "--------------------------------\n" );
     gm_log( GM_LOG_DEBUG, "configuration:\n" );
-    gm_log( GM_LOG_DEBUG, "log level:           %d\n", opt->debug_level);
+    gm_log( GM_LOG_DEBUG, "log level:                       %d\n", opt->debug_level);
 
     if(opt->logmode == GM_LOG_MODE_AUTO)
-        gm_log( GM_LOG_DEBUG, "log mode:            auto (%d)\n", opt->logmode);
+        gm_log( GM_LOG_DEBUG, "log mode:                        auto (%d)\n", opt->logmode);
     if(opt->logmode == GM_LOG_MODE_FILE)
-        gm_log( GM_LOG_DEBUG, "log mode:            file (%d)\n", opt->logmode);
+        gm_log( GM_LOG_DEBUG, "log mode:                        file (%d)\n", opt->logmode);
     if(opt->logmode == GM_LOG_MODE_STDOUT)
-        gm_log( GM_LOG_DEBUG, "log mode:            stdout (%d)\n", opt->logmode);
+        gm_log( GM_LOG_DEBUG, "log mode:                        stdout (%d)\n", opt->logmode);
     if(opt->logmode == GM_LOG_MODE_CORE)
-        gm_log( GM_LOG_DEBUG, "log mode:            core (%d)\n", opt->logmode);
+        gm_log( GM_LOG_DEBUG, "log mode:                        core (%d)\n", opt->logmode);
     if(opt->logmode == GM_LOG_MODE_SYSLOG)
-        gm_log( GM_LOG_DEBUG, "log mode:            syslog (%d)\n", opt->logmode);
+        gm_log( GM_LOG_DEBUG, "log mode:                        syslog (%d)\n", opt->logmode);
     if(opt->logmode == GM_LOG_MODE_TOOLS)
-        gm_log( GM_LOG_DEBUG, "log mode:            tools (%d)\n", opt->logmode);
+        gm_log( GM_LOG_DEBUG, "log mode:                        tools (%d)\n", opt->logmode);
 
     if(mode == GM_WORKER_MODE) {
-        gm_log( GM_LOG_DEBUG, "identifier:          %s\n", opt->identifier);
-        gm_log( GM_LOG_DEBUG, "pidfile:             %s\n", opt->pidfile == NULL ? "no" : opt->pidfile);
-        gm_log( GM_LOG_DEBUG, "logfile:             %s\n", opt->logfile == NULL ? "no" : opt->logfile);
-        gm_log( GM_LOG_DEBUG, "job max num:         %d\n", opt->max_jobs);
-        gm_log( GM_LOG_DEBUG, "job max age:         %d\n", opt->max_age);
-        gm_log( GM_LOG_DEBUG, "job timeout:         %d\n", opt->job_timeout);
-        gm_log( GM_LOG_DEBUG, "min worker:          %d\n", opt->min_worker);
-        gm_log( GM_LOG_DEBUG, "max worker:          %d\n", opt->max_worker);
-        gm_log( GM_LOG_DEBUG, "spawn rate:          %d\n", opt->spawn_rate);
-        gm_log( GM_LOG_DEBUG, "fork on exec:        %s\n", opt->fork_on_exec == GM_ENABLED ? "yes" : "no");
+        gm_log( GM_LOG_DEBUG, "identifier:                      %s\n", opt->identifier);
+        gm_log( GM_LOG_DEBUG, "pidfile:                         %s\n", opt->pidfile == NULL ? "no" : opt->pidfile);
+        gm_log( GM_LOG_DEBUG, "logfile:                         %s\n", opt->logfile == NULL ? "no" : opt->logfile);
+        gm_log( GM_LOG_DEBUG, "job max num:                     %d\n", opt->max_jobs);
+        gm_log( GM_LOG_DEBUG, "job max age:                     %d\n", opt->max_age);
+        gm_log( GM_LOG_DEBUG, "job timeout:                     %d\n", opt->job_timeout);
+        gm_log( GM_LOG_DEBUG, "min worker:                      %d\n", opt->min_worker);
+        gm_log( GM_LOG_DEBUG, "max worker:                      %d\n", opt->max_worker);
+        gm_log( GM_LOG_DEBUG, "spawn rate:                      %d\n", opt->spawn_rate);
+        gm_log( GM_LOG_DEBUG, "fork on exec:                    %s\n", opt->fork_on_exec == GM_ENABLED ? "yes" : "no");
 #ifndef EMBEDDEDPERL
-        gm_log( GM_LOG_DEBUG, "embedded perl:       not compiled\n");
+        gm_log( GM_LOG_DEBUG, "embedded perl:                   not compiled\n");
 #endif
 #ifdef EMBEDDEDPERL
         gm_log( GM_LOG_DEBUG, "\n" );
-        gm_log( GM_LOG_DEBUG, "embedded perl:       %s\n", opt->enable_embedded_perl == GM_ENABLED ? "yes" : "no");
-        gm_log( GM_LOG_DEBUG, "use_epn_implicitly:  %s\n", opt->use_embedded_perl_implicitly == GM_ENABLED ? "yes" : "no");
-        gm_log( GM_LOG_DEBUG, "use_perl_cache:      %s\n", opt->use_perl_cache == GM_ENABLED ? "yes" : "no");
-        gm_log( GM_LOG_DEBUG, "p1_file:             %s\n", opt->p1_file == NULL ? "not set" : opt->p1_file );
+        gm_log( GM_LOG_DEBUG, "embedded perl:                   %s\n", opt->enable_embedded_perl == GM_ENABLED ? "yes" : "no");
+        gm_log( GM_LOG_DEBUG, "use_epn_implicitly:              %s\n", opt->use_embedded_perl_implicitly == GM_ENABLED ? "yes" : "no");
+        gm_log( GM_LOG_DEBUG, "use_perl_cache:                  %s\n", opt->use_perl_cache == GM_ENABLED ? "yes" : "no");
+        gm_log( GM_LOG_DEBUG, "p1_file:                         %s\n", opt->p1_file == NULL ? "not set" : opt->p1_file );
 #endif
     }
     if(mode == GM_NEB_MODE) {
-        gm_log( GM_LOG_DEBUG, "queue by cust var:   %s\n", opt->queue_cust_var == NULL ? "no" : opt->queue_cust_var);
-        gm_log( GM_LOG_DEBUG, "debug result:        %s\n", opt->debug_result == GM_ENABLED ? "yes" : "no");
-        gm_log( GM_LOG_DEBUG, "result_worker:       %d\n", opt->result_workers);
-        gm_log( GM_LOG_DEBUG, "do_hostchecks:       %s\n", opt->do_hostchecks == GM_ENABLED ? "yes" : "no");
+        gm_log( GM_LOG_DEBUG, "queue by cust var:               %s\n", opt->queue_cust_var == NULL ? "no" : opt->queue_cust_var);
+        gm_log( GM_LOG_DEBUG, "debug result:                    %s\n", opt->debug_result == GM_ENABLED ? "yes" : "no");
+        gm_log( GM_LOG_DEBUG, "result_worker:                   %d\n", opt->result_workers);
+        gm_log( GM_LOG_DEBUG, "do_hostchecks:                   %s\n", opt->do_hostchecks == GM_ENABLED ? "yes" : "no");
+        gm_log( GM_LOG_DEBUG, "route_eventhandler_like_checks:  %s\n", opt->route_eventhandler_like_checks == GM_ENABLED ? "yes" : "no");
     }
     if(mode == GM_NEB_MODE || mode == GM_SEND_GEARMAN_MODE) {
-        gm_log( GM_LOG_DEBUG, "result_queue:        %s\n", opt->result_queue);
+        gm_log( GM_LOG_DEBUG, "result_queue:                    %s\n", opt->result_queue);
     }
     gm_log( GM_LOG_DEBUG, "\n" );
 
     /* server && queues */
     for(i=0;i<opt->server_num;i++)
-        gm_log( GM_LOG_DEBUG, "server:              %s:%i\n", opt->server_list[i]->host, opt->server_list[i]->port);
+        gm_log( GM_LOG_DEBUG, "server:                          %s:%i\n", opt->server_list[i]->host, opt->server_list[i]->port);
     gm_log( GM_LOG_DEBUG, "\n" );
     for(i=0;i<opt->dupserver_num;i++)
-        gm_log( GM_LOG_DEBUG, "dupserver:           %s:%i\n", opt->dupserver_list[i]->host, opt->dupserver_list[i]->port);
+        gm_log( GM_LOG_DEBUG, "dupserver:                       %s:%i\n", opt->dupserver_list[i]->host, opt->dupserver_list[i]->port);
     gm_log( GM_LOG_DEBUG, "\n" );
     if(mode == GM_NEB_MODE) {
-        gm_log( GM_LOG_DEBUG, "perfdata:            %s\n", opt->perfdata      == GM_ENABLED ? "yes" : "no");
-        gm_log( GM_LOG_DEBUG, "perfdata mode:       %s\n", opt->perfdata_mode == GM_PERFDATA_OVERWRITE ? "overwrite" : "append");
+        gm_log( GM_LOG_DEBUG, "perfdata:                        %s\n", opt->perfdata      == GM_ENABLED ? "yes" : "no");
+        gm_log( GM_LOG_DEBUG, "perfdata mode:                   %s\n", opt->perfdata_mode == GM_PERFDATA_OVERWRITE ? "overwrite" : "append");
     }
     if(mode == GM_NEB_MODE || mode == GM_WORKER_MODE) {
-        gm_log( GM_LOG_DEBUG, "hosts:               %s\n", opt->hosts        == GM_ENABLED ? "yes" : "no");
-        gm_log( GM_LOG_DEBUG, "services:            %s\n", opt->services     == GM_ENABLED ? "yes" : "no");
-        gm_log( GM_LOG_DEBUG, "eventhandler:        %s\n", opt->events       == GM_ENABLED ? "yes" : "no");
+        gm_log( GM_LOG_DEBUG, "hosts:                           %s\n", opt->hosts        == GM_ENABLED ? "yes" : "no");
+        gm_log( GM_LOG_DEBUG, "services:                        %s\n", opt->services     == GM_ENABLED ? "yes" : "no");
+        gm_log( GM_LOG_DEBUG, "eventhandler:                    %s\n", opt->events       == GM_ENABLED ? "yes" : "no");
         for(i=0;i<opt->hostgroups_num;i++)
-            gm_log( GM_LOG_DEBUG, "hostgroups:          %s\n", opt->hostgroups_list[i]);
+            gm_log( GM_LOG_DEBUG, "hostgroups:                      %s\n", opt->hostgroups_list[i]);
         for(i=0;i<opt->servicegroups_num;i++)
-            gm_log( GM_LOG_DEBUG, "servicegroups:       %s\n", opt->servicegroups_list[i]);
+            gm_log( GM_LOG_DEBUG, "servicegroups:                   %s\n", opt->servicegroups_list[i]);
     }
 
     if(mode == GM_NEB_MODE) {
         for(i=0;i<opt->local_hostgroups_num;i++)
-            gm_log( GM_LOG_DEBUG, "local_hostgroups: %s\n", opt->local_hostgroups_list[i]);
+            gm_log( GM_LOG_DEBUG, "local_hostgroups:                %s\n", opt->local_hostgroups_list[i]);
         for(i=0;i<opt->local_servicegroups_num;i++)
-            gm_log( GM_LOG_DEBUG, "local_servicegroups:      %s\n", opt->local_servicegroups_list[i]);
+            gm_log( GM_LOG_DEBUG, "local_servicegroups:             %s\n", opt->local_servicegroups_list[i]);
         /* export queues*/
         for(i=0;i<GM_NEBTYPESSIZE;i++) {
             char * type = nebcallback2str(i);
             for(j=0;j<opt->exports[i]->elem_number;j++)
-                gm_log( GM_LOG_DEBUG, "export:              %-45s -> %s\n", type, opt->exports[i]->name[j]);
+                gm_log( GM_LOG_DEBUG, "export:                          %-45s -> %s\n", type, opt->exports[i]->name[j]);
             free(type);
         }
     }
 
     /* encryption */
     gm_log( GM_LOG_DEBUG, "\n" );
-    gm_log( GM_LOG_DEBUG, "encryption:          %s\n", opt->encryption == GM_ENABLED ? "yes" : "no");
+    gm_log( GM_LOG_DEBUG, "encryption:                      %s\n", opt->encryption == GM_ENABLED ? "yes" : "no");
     if(opt->encryption == GM_ENABLED) {
-        gm_log( GM_LOG_DEBUG, "keyfile:             %s\n", opt->keyfile == NULL ? "no" : opt->keyfile);
+        gm_log( GM_LOG_DEBUG, "keyfile:                         %s\n", opt->keyfile == NULL ? "no" : opt->keyfile);
         if(opt->crypt_key != NULL) {
-            gm_log( GM_LOG_DEBUG, "encryption key:      set\n" );
+            gm_log( GM_LOG_DEBUG, "encryption key:                  set\n" );
         } else {
-            gm_log( GM_LOG_DEBUG, "encryption key:      not set\n" );
+            gm_log( GM_LOG_DEBUG, "encryption key:                  not set\n" );
         }
     }
     if(mode == GM_NEB_MODE) {
-        gm_log( GM_LOG_DEBUG, "accept clear result: %s\n", opt->accept_clear_results == GM_ENABLED ? "yes" : "no");
+        gm_log( GM_LOG_DEBUG, "accept clear result:             %s\n", opt->accept_clear_results == GM_ENABLED ? "yes" : "no");
     }
-    gm_log( GM_LOG_DEBUG, "transport mode:      %s\n", opt->encryption == GM_ENABLED ? "aes-256+base64" : "base64 only");
-    gm_log( GM_LOG_DEBUG, "use uniq jobs:       %s\n", opt->use_uniq_jobs == GM_ENABLED ? "overwrite" : "append");
+    gm_log( GM_LOG_DEBUG, "transport mode:                  %s\n", opt->encryption == GM_ENABLED ? "aes-256+base64" : "base64 only");
+    gm_log( GM_LOG_DEBUG, "use uniq jobs:                   %s\n", opt->use_uniq_jobs == GM_ENABLED ? "overwrite" : "append");
 
     gm_log( GM_LOG_DEBUG, "--------------------------------\n" );
     return;
@@ -1152,6 +1190,7 @@ int set_default_job(gm_job_t *job, mod_gm_opt_t *opt) {
     job->timeout             = opt->job_timeout;
     job->start_time.tv_sec   = 0L;
     job->start_time.tv_usec  = 0L;
+    job->has_been_sent       = FALSE;
 
     return(GM_OK);
 }
@@ -1648,6 +1687,12 @@ int check_param_server(gm_server_t * new_server, gm_server_t * server_list[GM_LI
 void send_result_back(gm_job_t * exec_job) {
     gm_log( GM_LOG_TRACE, "send_result_back()\n" );
 
+    /* avoid duplicate returned results */
+    if(exec_job->has_been_sent == TRUE) {
+        return;
+    }
+    exec_job->has_been_sent = TRUE;
+
     if(exec_job->result_queue == NULL) {
         return;
     }
