@@ -414,6 +414,11 @@ int set_worker( gearman_worker_st *w ) {
     return GM_OK;
 }
 
+/* called when worker runs into exit timeout */
+void exit_sighandler(int sig) {
+    gm_log( GM_LOG_TRACE, "exit_sighandler(%i)\n", sig );
+    _exit( EXIT_SUCCESS );
+}
 
 /* called when worker runs into idle timeout */
 void idle_sighandler(int sig) {
@@ -432,6 +437,10 @@ void set_state(int status) {
     if(worker_run_mode == GM_WORKER_STANDALONE)
         return;
 
+    /* give us 10 seconds to set state */
+    signal(SIGALRM, exit_sighandler);
+    alarm(10);
+
     /* Now we attach the segment to our data space. */
     if ((shm = shmat(shmid, NULL, 0)) == (int *) -1) {
         perror("shmat");
@@ -443,7 +452,7 @@ void set_state(int status) {
     if(status == GM_JOB_START)
         shm[shm_index] = current_pid;
     if(status == GM_JOB_END) {
-        shm[0]++; /* increase jobs done */
+        shm[SHM_JOBS_DONE]++; /* increase jobs done */
 
         /* status slot changed to -1 -> exit */
         if( shm[shm_index] == -1 ) {
@@ -465,6 +474,8 @@ void set_state(int status) {
     if(shmdt(shm) < 0)
         perror("shmdt");
 
+    alarm(0);
+
     return;
 }
 
@@ -473,11 +484,15 @@ void set_state(int status) {
 void clean_worker_exit(int sig) {
     int *shm;
 
+    /* give us 30 seconds to stop */
+    signal(SIGALRM, exit_sighandler);
+    alarm(30);
+
     gm_log( GM_LOG_TRACE, "clean_worker_exit(%d)\n", sig);
 
     /* clear gearmans job, otherwise it would be retried and retried */
     if(current_gearman_job != NULL) {
-        if(sig == 2) {
+        if(sig == SIGINT) {
             /* if worker stopped with sigint, let the job retry */
         } else {
             send_failed_result(current_job, sig);
@@ -507,6 +522,7 @@ void clean_worker_exit(int sig) {
         gm_log( GM_LOG_TRACE, "worker finished: %d\n", getpid() );
         _exit( EXIT_FAILURE );
     }
+    /* clean our pid from worker list */
     if( shm[shm_index] == current_pid || shm[shm_index] == -current_pid ) {
         shm[shm_index] = -1;
     }
@@ -545,22 +561,29 @@ void *return_status( gearman_job_st *job, void *context, size_t *result_size, ge
     result = malloc(GM_BUFFERSIZE);
     *result_size = GM_BUFFERSIZE;
 
+    /* give us 10 seconds to get state */
+    signal(SIGALRM, exit_sighandler);
+    alarm(10);
+
     /* Now we attach the segment to our data space. */
     if ((shm = shmat(shmid, NULL, 0)) == (int *) -1) {
         perror("shmat");
         *result_size = 0;
+        alarm(0);
         return NULL;
     }
 
-    snprintf(result, GM_BUFFERSIZE, "%s has %i worker and is working on %i jobs. Version: %s|worker=%i;;;%i;%i jobs=%ic", hostname, shm[1], shm[2], GM_VERSION, shm[1], mod_gm_opt->min_worker, mod_gm_opt->max_worker, shm[0] );
+    snprintf(result, GM_BUFFERSIZE, "%s has %i worker and is working on %i jobs. Version: %s|worker=%i;;;%i;%i jobs=%ic", hostname, shm[SHM_WORKER_TOTAL], shm[SHM_WORKER_RUNNING], GM_VERSION, shm[SHM_WORKER_TOTAL], mod_gm_opt->min_worker, mod_gm_opt->max_worker, shm[SHM_JOBS_DONE] );
 
     /* and increase job counter */
-    shm[0]++;
+    shm[SHM_JOBS_DONE]++;
 
     /* detach from shared memory */
     if(shmdt(shm) < 0)
         perror("shmdt");
 
+    alarm(0);
+
     return((void*)result);
 }
 
